<?php

namespace FiloBlu\Kpi\Model;
use FiloBlu\Kpi\Helper\Data;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\ResourceModel\Product\Action;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Customer\Model\ResourceModel\Customer\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Filesystem;
use Magento\Framework\Image\AdapterFactory;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Store\Model\StoreManagerInterface;

use function count;

/**
 * Class Core
 * @package FiloBlu\Kpi\Model
 */
class Core
{
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $_productCollection;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var SerializerInterface
     */
    protected $serializer;
    /**
     * @var ResourceConnection
     */
    protected $_resource;
    /**
     * @var CollectionFactory
     */
    protected $_customerFactory;

    /**
     * @var ProductRepositoryInterface
     */
    protected $_productRepositoryInterface;

    /**
     * @var Configurable
     */
    protected $_productConfigurable;
    /**
     * @var Filesystem
     */
    protected $_filesystem;
    /**
     * @var AdapterFactory
     */
    protected $_adapterFactory;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManagerInterface;
    /**
     * @var Action
     */
    protected $_productAction;
    /**
     * @var
     */
    protected $connection;

    /**
     * @var Data
     */
    protected $_helper;

    /**
     * Core constructor.
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection
     * @param ScopeConfigInterface $scopeConfig
     * @param SerializerInterface $serializer
     * @param ResourceConnection $resource
     * @param CollectionFactory $customerFactory
     * @param ProductRepositoryInterface $productRepositoryInterface
     * @param Data $helper
     * @param Configurable $productConfigurable
     * @param Filesystem $filesystem
     * @param AdapterFactory $adapterFactory
     * @param StoreManagerInterface $storeManagerInterface
     * @param Action $productAction
     */
    public function __construct(
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection,
        ScopeConfigInterface $scopeConfig,
        SerializerInterface $serializer,
        ResourceConnection $resource,
        CollectionFactory $customerFactory,
        ProductRepositoryInterface $productRepositoryInterface,
        Data $helper,
        Configurable $productConfigurable,
        Filesystem $filesystem,
        AdapterFactory $adapterFactory,
        StoreManagerInterface $storeManagerInterface,
        Action $productAction
    )
    {
        $this->_productCollection = $productCollection;
        $this->_scopeConfig = $scopeConfig;
        $this->serializer = $serializer;
        $this->_resource = $resource;
        $this->_customerFactory = $customerFactory;
        $this->_productRepositoryInterface = $productRepositoryInterface;
        $this->_helper = $helper;
        $this->_productConfigurable = $productConfigurable;
        $this->_filesystem = $filesystem;
        $this->_adapterFactory = $adapterFactory;
        $this->_storeManagerInterface = $storeManagerInterface;
        $this->_productAction = $productAction;
    }

    /**
     * @return AdapterInterface
     */
    protected function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_write');
        }
        return $this->connection;
    }

    /**
     * @return mixed
     */
    protected function getCustomerCollection()
    {
        return $this->_customerFactory->create();
    }

    /**
     * @return mixed
     */
    protected function getProductsCollection()
    {
        return $this->_productCollection->create();
    }

    /**
     * @param bool $get_only_first
     * @return array|mixed
     */
    protected function getDefaultStoreIds($get_only_first = false)
    {
        $result = [];
        $fallback_to_store_id = 1;

        $affected_stores = trim((string)$this->_scopeConfig->getValue('kpi/table_configuration/affectedstores'));

        // Fallback store ID 1
        if (!$affected_stores) {
            $result[] = $fallback_to_store_id;

            if($get_only_first){
                return $fallback_to_store_id;
            }

            return $result;
        }

        $store_codes = explode(',', $affected_stores);
        $store_codes = array_map(static function ($code) { return trim((string)$code);} , $store_codes);

        $all_stores = $this->_storeManagerInterface->getStores();

        foreach ($store_codes as $store_code) {
            foreach ($all_stores as $store) {
                if ($store->getCode() == $store_code) {
                    $result[] = $store->getId();
                }
            }
        }

        // Fallback to default store 1 if none is found
        if (count($result) === 0) {
            $result = [];
            $result[] = $fallback_to_store_id;
        }

        if ($get_only_first) {
            return $result[0];
        }

        return $result;
    }

}
